(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var ECMAScript = Package.ecmascript.ECMAScript;
var _ = Package.underscore._;
var Random = Package.random.Random;
var Router = Package['iron:router'].Router;
var RouteController = Package['iron:router'].RouteController;
var SandstormDb = Package['sandstorm-db'].SandstormDb;
var SandstormEmail = Package['sandstorm-email'].SandstormEmail;
var Accounts = Package['accounts-base'].Accounts;
var AccountsServer = Package['accounts-base'].AccountsServer;
var check = Package.check.check;
var Match = Package.check.Match;
var SHA256 = Package.sha.SHA256;
var Email = Package.email.Email;
var EmailInternals = Package.email.EmailInternals;
var babelHelpers = Package['babel-runtime'].babelHelpers;
var Symbol = Package['ecmascript-runtime'].Symbol;
var Map = Package['ecmascript-runtime'].Map;
var Set = Package['ecmascript-runtime'].Set;
var Promise = Package.promise.Promise;
var Iron = Package['iron:core'].Iron;

(function(){

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                               //
// packages/accounts-email-token/token_common.js                                                                 //
//                                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                 //
Accounts.emailToken = {};                                                                                        // 1
                                                                                                                 //
Accounts.emailToken._hashToken = function (token) {                                                              // 3
  return {                                                                                                       // 4
    digest: SHA256(token),                                                                                       // 5
    algorithm: "sha-256"                                                                                         // 6
  };                                                                                                             //
};                                                                                                               //
                                                                                                                 //
Router.route("/_emailLogin/:_email/:_token", function () {                                                       // 10
  var _this = this;                                                                                              //
                                                                                                                 //
  this.render("Loading");                                                                                        // 11
  Meteor.loginWithEmailToken(this.params._email, this.params._token, function (err) {                            // 12
    if (err) {                                                                                                   // 13
      _this.render("_emailTokenError", {                                                                         // 14
        data: function () {                                                                                      // 15
          return {                                                                                               // 16
            error: err                                                                                           // 17
          };                                                                                                     //
        }                                                                                                        //
      });                                                                                                        //
    } else {                                                                                                     //
      Router.go("/");                                                                                            // 22
    }                                                                                                            //
  });                                                                                                            //
}, {                                                                                                             //
  // Specify a name in an attempt to fix bug reported here:                                                      //
  //   https://twitter.com/fink_/status/709715523513270272                                                       //
  //                                                                                                             //
  // The user reported an exception trace where IronRouter complained that the route "i" had been                //
  // registered twice, tracing back to routes in this file. Where did "i" come from? The problem                 //
  // does not seem to be reproducible.                                                                           //
  //                                                                                                             //
  // It looks like IronRouter may decide to name the route based on the function name:                           //
  //   https://github.com/iron-meteor/iron-middleware-stack/blob/master/lib/handler.js#L49                       //
  //                                                                                                             //
  // Our function has no name, but perhaps minification can mess with that? An explicit name                     //
  // in the options will take precedence, so do that.                                                            //
  name: "_emailLogin"                                                                                            // 38
});                                                                                                              //
                                                                                                                 //
Router.route("/_emailLinkIdentity/:_email/:_token/:_accountId", function () {                                    // 41
  var _this2 = this;                                                                                             //
                                                                                                                 //
  this.render("Loading");                                                                                        // 42
  if (this.state.get("error")) {                                                                                 // 43
    this.render("_emailLinkIdentityError", { data: { error: this.state.get("error") } });                        // 44
  } else {                                                                                                       //
    if (Meteor.userId() === this.params._accountId) {                                                            // 46
      Meteor.call("linkEmailIdentityToAccount", this.params._email, this.params._token, function (err, result) {
        if (err) {                                                                                               // 49
          _this2.state.set("error", err.toString());                                                             // 50
        } else {                                                                                                 //
          Router.go("/account");                                                                                 // 52
        }                                                                                                        //
      });                                                                                                        //
    } else {                                                                                                     //
      this.render("_emailLinkIdentityError");                                                                    // 56
    }                                                                                                            //
  }                                                                                                              //
}, {                                                                                                             //
  // See above.                                                                                                  //
  name: "_emailLinkIdentity"                                                                                     // 61
});                                                                                                              //
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);






(function(){

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                               //
// packages/accounts-email-token/token_server.js                                                                 //
//                                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                 //
var TOKEN_EXPIRATION_MS = 15 * 60 * 1000;                                                                        // 1
                                                                                                                 //
var cleanupExpiredTokens = function () {                                                                         // 3
  Meteor.users.update({                                                                                          // 4
    "services.email.tokens.createdAt": {                                                                         // 5
      $lt: new Date(Date.now() - TOKEN_EXPIRATION_MS)                                                            // 6
    }                                                                                                            //
  }, {                                                                                                           //
    $pull: {                                                                                                     // 9
      "services.email.tokens": {                                                                                 // 10
        createdAt: { $lt: new Date(Date.now() - TOKEN_EXPIRATION_MS) }                                           // 11
      }                                                                                                          //
    }                                                                                                            //
  }, {                                                                                                           //
    multi: true                                                                                                  // 15
  });                                                                                                            //
};                                                                                                               //
                                                                                                                 //
Meteor.startup(cleanupExpiredTokens);                                                                            // 19
// Tokens can actually last up to 2 * TOKEN_EXPIRATION_MS                                                        //
SandstormDb.periodicCleanup(TOKEN_EXPIRATION_MS, cleanupExpiredTokens);                                          // 21
                                                                                                                 //
var checkToken = function (tokens, token) {                                                                      // 23
  var found = false;                                                                                             // 24
  tokens.forEach(function (userToken) {                                                                          // 25
    if (userToken.algorithm === token.algorithm && userToken.digest === token.digest) {                          // 26
      found = true;                                                                                              // 28
    }                                                                                                            //
  });                                                                                                            //
                                                                                                                 //
  return found;                                                                                                  // 32
};                                                                                                               //
                                                                                                                 //
function consumeToken(user, token) {                                                                             // 35
  var hashedToken = Accounts.emailToken._hashToken(token);                                                       // 36
  var found = checkToken(user.services.email.tokens, hashedToken);                                               // 37
                                                                                                                 //
  if (found) {                                                                                                   // 39
    Meteor.users.update({ _id: user._id }, { $pull: { "services.email.tokens": hashedToken } });                 // 40
  }                                                                                                              //
                                                                                                                 //
  return found;                                                                                                  // 43
}                                                                                                                //
                                                                                                                 //
// Handler to login with a token.                                                                                //
Accounts.registerLoginHandler("email", function (options) {                                                      // 47
  if (!options.email) {                                                                                          // 48
    return undefined; // don't handle                                                                            // 49
  }                                                                                                              //
                                                                                                                 //
  if (!Accounts.identityServices.email.isEnabled()) {                                                            // 52
    throw new Meteor.Error(403, "Email identity service is disabled.");                                          // 53
  }                                                                                                              //
                                                                                                                 //
  options = options.email;                                                                                       // 56
  check(options, {                                                                                               // 57
    email: String,                                                                                               // 58
    token: String                                                                                                // 59
  });                                                                                                            //
                                                                                                                 //
  var user = Meteor.users.findOne({                                                                              // 62
    "services.email.email": options.email                                                                        // 63
  }, {                                                                                                           //
    fields: {                                                                                                    // 65
      "services.email": 1                                                                                        // 66
    }                                                                                                            //
  });                                                                                                            //
                                                                                                                 //
  if (!user) {                                                                                                   // 70
    console.error("User not found:", options.email);                                                             // 71
    return {                                                                                                     // 72
      error: new Meteor.Error(403, "User not found")                                                             // 73
    };                                                                                                           //
  }                                                                                                              //
                                                                                                                 //
  if (!user.services.email.tokens) {                                                                             // 77
    console.error("User has no token set:", options.email);                                                      // 78
    return {                                                                                                     // 79
      error: new Meteor.Error(403, "User has no token set")                                                      // 80
    };                                                                                                           //
  }                                                                                                              //
                                                                                                                 //
  if (!consumeToken(user, options.token.trim())) {                                                               // 84
    console.error("Token not found:", options.email);                                                            // 85
    return {                                                                                                     // 86
      error: new Meteor.Error(403, "Invalid authentication code")                                                // 87
    };                                                                                                           //
  }                                                                                                              //
                                                                                                                 //
  return {                                                                                                       // 91
    userId: user._id                                                                                             // 92
  };                                                                                                             //
});                                                                                                              //
                                                                                                                 //
var Url = Npm.require("url");                                                                                    // 96
                                                                                                                 //
var ROOT_URL = Url.parse(process.env.ROOT_URL);                                                                  // 98
var HOSTNAME = ROOT_URL.hostname;                                                                                // 99
                                                                                                                 //
var makeTokenUrl = function (email, token, linkingIdentity) {                                                    // 101
  if (linkingIdentity) {                                                                                         // 102
    return process.env.ROOT_URL + "/_emailLinkIdentity/" + encodeURIComponent(email) + "/" + encodeURIComponent(token) + "/" + Meteor.userId();
  } else {                                                                                                       //
    return process.env.ROOT_URL + "/_emailLogin/" + encodeURIComponent(email) + "/" + encodeURIComponent(token);
  }                                                                                                              //
};                                                                                                               //
                                                                                                                 //
///                                                                                                              //
/// EMAIL VERIFICATION                                                                                           //
///                                                                                                              //
var sendTokenEmail = function (db, email, token, linkingIdentity) {                                              // 114
  var subject = undefined;                                                                                       // 115
  var text = undefined;                                                                                          // 116
  if (!linkingIdentity) {                                                                                        // 117
    subject = "Log in to " + HOSTNAME;                                                                           // 118
    text = "To confirm this email address on ";                                                                  // 119
  } else {                                                                                                       //
    subject = "Confirm this email address on " + HOSTNAME;                                                       // 121
    text = "To confirm this email address on ";                                                                  // 122
  }                                                                                                              //
                                                                                                                 //
  text = text + HOSTNAME + ", click on the following link:\n\n" + makeTokenUrl(email, token, linkingIdentity) + "\n\n" + "Alternatively, enter the following one-time authentication code into the log-in form:\n\n" + token;
                                                                                                                 //
  var options = {                                                                                                // 130
    to: email,                                                                                                   // 131
    from: db.getServerTitle() + " <" + db.getReturnAddress() + ">",                                              // 132
    subject: subject,                                                                                            // 133
    text: text                                                                                                   // 134
  };                                                                                                             //
                                                                                                                 //
  SandstormEmail.send(options);                                                                                  // 137
};                                                                                                               //
                                                                                                                 //
///                                                                                                              //
/// CREATING USERS                                                                                               //
///                                                                                                              //
// returns the user id                                                                                           //
var createAndEmailTokenForUser = function (db, email, linkingIdentity) {                                         // 144
  check(email, String);                                                                                          // 145
  check(linkingIdentity, Boolean);                                                                               // 146
  var atIndex = email.indexOf("@");                                                                              // 147
  if (atIndex === -1) {                                                                                          // 148
    throw new Meteor.Error(400, "No @ symbol was found in your email");                                          // 149
  }                                                                                                              //
                                                                                                                 //
  var user = Meteor.users.findOne({ "services.email.email": email }, { fields: { "services.email": 1 } });       // 152
  var userId = undefined;                                                                                        // 154
                                                                                                                 //
  // TODO(someday): make this shorter, and handle requests that try to brute force it.                           //
  var token = Random.id(12);                                                                                     // 157
  var tokenObj = Accounts.emailToken._hashToken(token);                                                          // 158
  tokenObj.createdAt = new Date();                                                                               // 159
                                                                                                                 //
  if (user) {                                                                                                    // 161
    if (user.services.email.tokens && user.services.email.tokens.length > 2) {                                   // 162
      throw new Meteor.Error(409, "It looks like we sent a log in email to this address not long " + "ago. Please use the one that was already sent (check your spam folder if you can't find " + "it), or wait a while and try again");
    }                                                                                                            //
                                                                                                                 //
    userId = user._id;                                                                                           // 168
                                                                                                                 //
    Meteor.users.update({ _id: user._id }, { $push: { "services.email.tokens": tokenObj } });                    // 170
  } else {                                                                                                       //
    var options = {};                                                                                            // 172
    user = {                                                                                                     // 173
      services: {                                                                                                // 174
        email: {                                                                                                 // 175
          tokens: [tokenObj],                                                                                    // 176
          email: email                                                                                           // 177
        }                                                                                                        //
      }                                                                                                          //
    };                                                                                                           //
                                                                                                                 //
    userId = Accounts.insertUserDoc(options, user);                                                              // 182
  }                                                                                                              //
                                                                                                                 //
  sendTokenEmail(db, email, token, linkingIdentity);                                                             // 185
                                                                                                                 //
  return userId;                                                                                                 // 187
};                                                                                                               //
                                                                                                                 //
Meteor.methods({                                                                                                 // 190
  createAndEmailTokenForUser: function (email, linkingIdentity) {                                                // 191
    // method for create user. Requests come from the client.                                                    //
    // This method will create a user if it doesn't exist, otherwise it will generate a token.                   //
    // It will always send an email to the user                                                                  //
                                                                                                                 //
    check(email, String);                                                                                        // 196
    check(linkingIdentity, Boolean);                                                                             // 197
                                                                                                                 //
    if (!Accounts.identityServices.email.isEnabled()) {                                                          // 199
      throw new Meteor.Error(403, "Email identity service is disabled.");                                        // 200
    }                                                                                                            //
    // Create user. result contains id and token.                                                                //
    var user = createAndEmailTokenForUser(this.connection.sandstormDb, email, linkingIdentity);                  // 203
  },                                                                                                             //
                                                                                                                 //
  linkEmailIdentityToAccount: function (email, token) {                                                          // 206
    // Links the email identity with address `email` and login token `token` to the current account.             //
    check(email, String);                                                                                        // 208
    check(token, String);                                                                                        // 209
    var account = Meteor.user();                                                                                 // 210
    if (!account || !account.loginIdentities) {                                                                  // 211
      throw new Meteor.Error(403, "Must be logged in to an account to link an email identity.");                 // 212
    }                                                                                                            //
                                                                                                                 //
    var identity = Meteor.users.findOne({ "services.email.email": email }, { fields: { "services.email": 1 } });
    if (!identity) {                                                                                             // 217
      throw new Meteor.Error(403, "Invalid authentication code.");                                               // 218
    }                                                                                                            //
                                                                                                                 //
    if (!consumeToken(identity, token)) {                                                                        // 221
      throw new Meteor.Error(403, "Invalid authentication code.");                                               // 222
    }                                                                                                            //
                                                                                                                 //
    Accounts.linkIdentityToAccount(this.connection.sandstormDb, this.connection.sandstormBackend, identity._id, account._id);
  }                                                                                                              //
});                                                                                                              //
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['accounts-email-token'] = {};

})();

//# sourceMappingURL=accounts-email-token.js.map
