(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var check = Package.check.check;
var Match = Package.check.Match;
var Random = Package.random.Random;
var _ = Package.underscore._;
var SandstormDb = Package['sandstorm-db'].SandstormDb;

/* Package-scope variables */
var SandstormPermissions, path;

(function(){

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                         //
// packages/sandstorm-permissions/permissions.js                                                           //
//                                                                                                         //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                           //
// Sandstorm - Personal Cloud Sandbox                                                                      // 1
// Copyright (c) 2015 Sandstorm Development Group, Inc. and contributors                                   // 2
// All rights reserved.                                                                                    // 3
//                                                                                                         // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                         // 5
// you may not use this file except in compliance with the License.                                        // 6
// You may obtain a copy of the License at                                                                 // 7
//                                                                                                         // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                            // 9
//                                                                                                         // 10
// Unless required by applicable law or agreed to in writing, software                                     // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                       // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                // 13
// See the License for the specific language governing permissions and                                     // 14
// limitations under the License.                                                                          // 15
                                                                                                           // 16
// Until David is done working on this:                                                                    // 17
// jscs:disable                                                                                            // 18
                                                                                                           // 19
var Crypto = Npm.require("crypto");                                                                        // 20
var Url = Npm.require("url");                                                                              // 21
                                                                                                           // 22
SandstormPermissions = {};                                                                                 // 23
                                                                                                           // 24
function PermissionSet(array) {                                                                            // 25
  // A wrapper around an array of booleans.                                                                // 26
                                                                                                           // 27
  if (!array) {                                                                                            // 28
    this.array = [];                                                                                       // 29
  } else if (array instanceof Array) {                                                                     // 30
    this.array = array.slice(0);                                                                           // 31
  } else {                                                                                                 // 32
    throw new Error("don't know how to interpret as PermissionSet: " + array);                             // 33
  }                                                                                                        // 34
}                                                                                                          // 35
                                                                                                           // 36
// Methods for mutating a PermissionSet by combining it with another PermissionSet.                        // 37
// These return a boolean indicating whether the operation had any effect.                                 // 38
                                                                                                           // 39
PermissionSet.prototype.add = function(other) {                                                            // 40
  var changed = false;                                                                                     // 41
  for (var ii = 0; ii < other.array.length; ++ii) {                                                        // 42
    var old = this.array[ii];                                                                              // 43
    this.array[ii] = this.array[ii] || other.array[ii];                                                    // 44
    if (old != this.array[ii]) {                                                                           // 45
      changed = true;                                                                                      // 46
    }                                                                                                      // 47
  }                                                                                                        // 48
  return changed;                                                                                          // 49
}                                                                                                          // 50
                                                                                                           // 51
PermissionSet.prototype.remove = function(other) {                                                         // 52
  var changed = false;                                                                                     // 53
  for (var ii = 0; ii < other.array.length && ii < this.array.length; ++ii) {                              // 54
    var old = this.array[ii];                                                                              // 55
    this.array[ii] = this.array[ii] && !other.array[ii];                                                   // 56
    if (old != this.array[ii]) {                                                                           // 57
      changed = true;                                                                                      // 58
    }                                                                                                      // 59
  }                                                                                                        // 60
  return changed;                                                                                          // 61
}                                                                                                          // 62
                                                                                                           // 63
PermissionSet.prototype.intersect = function(other) {                                                      // 64
  var changed = false;                                                                                     // 65
  for (var ii = 0; ii < this.array.length; ++ii) {                                                         // 66
    var old = this.array[ii];                                                                              // 67
    this.array[ii] = this.array[ii] && other.array[ii];                                                    // 68
    if (old != this.array[ii]) {                                                                           // 69
      changed = true;                                                                                      // 70
    }                                                                                                      // 71
  }                                                                                                        // 72
  return changed;                                                                                          // 73
}                                                                                                          // 74
                                                                                                           // 75
function roleAssignmentPermissions(roleAssignment, viewInfo) {                                             // 76
  var result = new PermissionSet([]);                                                                      // 77
                                                                                                           // 78
  if (!roleAssignment || "none" in roleAssignment) {                                                       // 79
    if (viewInfo.roles) {                                                                                  // 80
      for (var ii = 0; ii < viewInfo.roles.length; ++ii) {                                                 // 81
        var roleDef = viewInfo.roles[ii];                                                                  // 82
        if (roleDef.default) {                                                                             // 83
          result = new PermissionSet(roleDef.permissions);                                                 // 84
          break;                                                                                           // 85
        }                                                                                                  // 86
      }                                                                                                    // 87
    }                                                                                                      // 88
  } else if ("allAccess" in roleAssignment) {                                                              // 89
    var length = 0;                                                                                        // 90
    if (viewInfo.permissions) {                                                                            // 91
      length = viewInfo.permissions.length;                                                                // 92
    }                                                                                                      // 93
    var array = new Array(length);                                                                         // 94
    for (var ii = 0; ii < array.length; ++ii) {                                                            // 95
      array[ii] = true;                                                                                    // 96
    }                                                                                                      // 97
    result = new PermissionSet(array);                                                                     // 98
  } else if ("roleId" in roleAssignment && viewInfo.roles && viewInfo.roles.length > 0) {                  // 99
    var roleDef = viewInfo.roles[roleAssignment.roleId];                                                   // 100
    if (roleDef) {                                                                                         // 101
      result = new PermissionSet(roleDef.permissions);                                                     // 102
    }                                                                                                      // 103
  }                                                                                                        // 104
                                                                                                           // 105
  if (roleAssignment) {                                                                                    // 106
    result.add(new PermissionSet(roleAssignment.addPermissionSet));                                        // 107
    result.remove(new PermissionSet(roleAssignment.removePermissionSet));                                  // 108
  }                                                                                                        // 109
  return result;                                                                                           // 110
}                                                                                                          // 111
                                                                                                           // 112
function collectEdges(db, vertex) {                                                                        // 113
  // Given a vertex in the sharing graph in the format specified by the `check()` invocation below,        // 114
  // collects the data needed for permissions computations pertaining to that vertex. There are three      // 115
  // self-explanatory special cases for the return value. In order of decreasing precedence, they          // 116
  // are: `{grainDoesNotExist: true}`, `{grainIsPublic: true}`, and `{disallowedAnonymousAccess: true}`.   // 117
  // In all other cases, this function returns an object of the form:                                      // 118
  //   `{edgesByRecipient: <object>, terminalEdge: Optional(<object>)}`.                                   // 119
  // The `edgesByRecipient` field is a map that coalesces chains of `parentToken` UiView tokens into       // 120
  // direct user-to-user edges; its keys are identity IDs of recipient users and its values are lists      // 121
  // of "edge" objects of the form                                                                         // 122
  //   `{sharer: OneOf(<identityId>, "OwningAccount"), roleAssignments: <list of role assignments>}`.      // 123
  // The role assignments should be applied in sequence to compute the set of permissions that flow to a   // 124
  // recipient from a sharer. The `terminalEdge` field is an edge object representing the link to          // 125
  // `vertex` from the nearest user in the sharing graph. If `vertex` is already a user, then this edge is
  // trivial and its `roleAssignments` field is an empty list. If `terminalEdge` is not present,           // 127
  // then there is no such link.                                                                           // 128
  //                                                                                                       // 129
  // TODO(now): UiView tokens can have membrane requirements; we need to account for them in this          // 130
  // computation.                                                                                          // 131
  check(vertex,                                                                                            // 132
        Match.OneOf({token: Match.ObjectIncluding({_id: String, grainId: String})},                        // 133
                    {grain: Match.ObjectIncluding(                                                         // 134
                      {_id: String,                                                                        // 135
                       identityId: Match.OneOf(String, null, undefined)})}));                              // 136
                                                                                                           // 137
  var grainId;                                                                                             // 138
  if (vertex.token) {                                                                                      // 139
    grainId = vertex.token.grainId;                                                                        // 140
  } else if (vertex.grain) {                                                                               // 141
    grainId = vertex.grain._id;                                                                            // 142
  }                                                                                                        // 143
  var grain = db.getGrain(grainId);                                                                        // 144
  if (!grain) {                                                                                            // 145
    return {grainDoesNotExist: true};                                                                      // 146
  }                                                                                                        // 147
                                                                                                           // 148
  if (!grain.private) {                                                                                    // 149
    return {grainIsPublic: true};                                                                          // 150
  } else if (vertex.grain && !vertex.grain.identityId) {                                                   // 151
    return {disallowedAnonymousAccess: true};                                                              // 152
  }                                                                                                        // 153
                                                                                                           // 154
  var result = {edgesByRecipient: {}};                                                                     // 155
                                                                                                           // 156
  var tokensById = {};                                                                                     // 157
  db.collections.apiTokens.find({grainId: grainId,                                                         // 158
                                 revoked: {$ne: true}}).forEach(function(token) {                          // 159
    tokensById[token._id] = token;                                                                         // 160
  });                                                                                                      // 161
                                                                                                           // 162
  function computeEdge(token) {                                                                            // 163
    var roleAssignments = [];                                                                              // 164
    var curToken = token;                                                                                  // 165
    while (curToken && curToken.parentToken) {                                                             // 166
      // For a `parentToken` provider, no role assignment means don't attenuate.                           // 167
      if (curToken.roleAssignment) {                                                                       // 168
        roleAssignments.push(curToken.roleAssignment);                                                     // 169
      }                                                                                                    // 170
      curToken = tokensById[curToken.parentToken];                                                         // 171
    }                                                                                                      // 172
    if (curToken && curToken.grainId && curToken.identityId) {                                             // 173
      roleAssignments.push(curToken.roleAssignment);                                                       // 174
      return {sharer: curToken.identityId, roleAssignments: roleAssignments};                              // 175
    } else {                                                                                               // 176
      return;                                                                                              // 177
    }                                                                                                      // 178
  }                                                                                                        // 179
                                                                                                           // 180
  if (vertex.token) {                                                                                      // 181
    result.terminalEdge = computeEdge(tokensById[vertex.token._id]);                                       // 182
  } else if (vertex.grain) {                                                                               // 183
    result.terminalEdge = {sharer: vertex.grain.identityId, roleAssignments: []};                          // 184
  }                                                                                                        // 185
                                                                                                           // 186
  for (var id in tokensById) {                                                                             // 187
    var token = tokensById[id];                                                                            // 188
    if (Match.test(token.owner, {user: Match.Any})) {                                                      // 189
      var edge = computeEdge(token);                                                                       // 190
      if (edge) {                                                                                          // 191
        var recipient = token.owner.user.identityId ;                                                      // 192
        if (!result.edgesByRecipient[recipient]) {                                                         // 193
          result.edgesByRecipient[recipient] = [];                                                         // 194
        }                                                                                                  // 195
        result.edgesByRecipient[recipient].push(edge);                                                     // 196
      }                                                                                                    // 197
    }                                                                                                      // 198
  }                                                                                                        // 199
                                                                                                           // 200
  var owningUser = Meteor.users.findOne({_id: grain.userId});                                              // 201
  if (owningUser) {                                                                                        // 202
    SandstormDb.getUserIdentityIds(owningUser).forEach(function(identityId) {                              // 203
      result.edgesByRecipient[identityId] = [{sharer: "OwningAccount", roleAssignments: []}];              // 204
    });                                                                                                    // 205
  }                                                                                                        // 206
                                                                                                           // 207
  return result;                                                                                           // 208
}                                                                                                          // 209
                                                                                                           // 210
SandstormPermissions.grainPermissions = function(db, vertex, viewInfo) {                                   // 211
  // Computes the permissions of a vertex. If the vertex is not allowed to open the grain,                 // 212
  // returns null. Otherwise, returns an array of bools representing the permissions held.                 // 213
  var edges = collectEdges(db, vertex);                                                                    // 214
  if (edges.openerIsOwner) {                                                                               // 215
    return roleAssignmentPermissions({allAccess: null}, viewInfo).array;                                   // 216
  }                                                                                                        // 217
  if (edges.grainIsPublic) {                                                                               // 218
    // Grains using the old sharing model always share the default role to anyone who has the              // 219
    // grain URL.                                                                                          // 220
    return roleAssignmentPermissions({none: null}, viewInfo).array;                                        // 221
  }                                                                                                        // 222
  if (edges.grainDoesNotExist || !edges.terminalEdge || edges.disallowedAnonymousAccess) {                 // 223
    return null;                                                                                           // 224
  }                                                                                                        // 225
                                                                                                           // 226
  var openerIdentityId = edges.terminalEdge.sharer;                                                        // 227
  var edgesByRecipient = edges.edgesByRecipient;                                                           // 228
  var owner = "OwningAccount";                                                                             // 229
                                                                                                           // 230
  var permissionsMap = {};                                                                                 // 231
  // Keeps track of the permissions that the opener receives from each user. The final result of           // 232
  // our computation will be stored in permissionsMap[owner].                                              // 233
                                                                                                           // 234
  var userStack = [openerIdentityId];                                                                      // 235
                                                                                                           // 236
  var openerAttenuation = roleAssignmentPermissions({allAccess: null}, viewInfo);                          // 237
  edges.terminalEdge.roleAssignments.forEach(function(roleAssignment) {                                    // 238
    openerAttenuation.intersect(roleAssignmentPermissions(roleAssignment, viewInfo));                      // 239
  });                                                                                                      // 240
  permissionsMap[openerIdentityId] = openerAttenuation;                                                    // 241
                                                                                                           // 242
  while (userStack.length > 0) {                                                                           // 243
    var recipient = userStack.pop();                                                                       // 244
    if (edgesByRecipient[recipient]) {                                                                     // 245
      edgesByRecipient[recipient].forEach(function (inEdge) {                                              // 246
        var sharer = inEdge.sharer;                                                                        // 247
        var needToPush = false;                                                                            // 248
        if (!permissionsMap[sharer]) {                                                                     // 249
          permissionsMap[sharer] = new PermissionSet();                                                    // 250
          needToPush = true;                                                                               // 251
        }                                                                                                  // 252
                                                                                                           // 253
        var newPermissions = roleAssignmentPermissions({allAccess: null}, viewInfo);                       // 254
        inEdge.roleAssignments.forEach(function(roleAssignment) {                                          // 255
          newPermissions.intersect(roleAssignmentPermissions(roleAssignment, viewInfo));                   // 256
        });                                                                                                // 257
        newPermissions.intersect(permissionsMap[recipient]);                                               // 258
                                                                                                           // 259
        // Optimization: we don't care about permissions that we've already proven the opener has.         // 260
        if (permissionsMap[owner]) {                                                                       // 261
          newPermissions.remove(permissionsMap[owner]);                                                    // 262
        }                                                                                                  // 263
                                                                                                           // 264
        if (permissionsMap[sharer].add(newPermissions)) {                                                  // 265
          needToPush = true;                                                                               // 266
        }                                                                                                  // 267
        if (needToPush) {                                                                                  // 268
          userStack.push(sharer);                                                                          // 269
        }                                                                                                  // 270
      });                                                                                                  // 271
    }                                                                                                      // 272
  }                                                                                                        // 273
  if (permissionsMap[owner]) {                                                                             // 274
    return permissionsMap[owner].array;                                                                    // 275
  } else {                                                                                                 // 276
    return null;                                                                                           // 277
  }                                                                                                        // 278
}                                                                                                          // 279
                                                                                                           // 280
SandstormPermissions.mayOpenGrain = function(db, vertex) {                                                 // 281
  // Determines whether the vertex is allowed to open the grain by searching depth first                   // 282
  // for a path of active role assignments leading from the grain owner to the vertex.                     // 283
                                                                                                           // 284
  var edges = collectEdges(db, vertex);                                                                    // 285
  if (edges.grainDoesNotExist || edges.disallowedAnonymousAccess) {                                        // 286
    return false;                                                                                          // 287
  }                                                                                                        // 288
  if (edges.grainIsPublic) {                                                                               // 289
    return true;                                                                                           // 290
  }                                                                                                        // 291
  var edgesByRecipient = edges.edgesByRecipient;                                                           // 292
  var owner = "OwningAccount";                                                                             // 293
  if (!edges.terminalEdge) { return false; }                                                               // 294
  if (owner == edges.terminalEdge.sharer) {                                                                // 295
    return true;                                                                                           // 296
  }                                                                                                        // 297
                                                                                                           // 298
  var openerIdentityId = edges.terminalEdge.sharer;                                                        // 299
  var stackedUsers = {};                                                                                   // 300
  stackedUsers[openerIdentityId] = true;                                                                   // 301
  var userStack = [openerIdentityId];                                                                      // 302
                                                                                                           // 303
  while (userStack.length > 0) {                                                                           // 304
    var recipient = userStack.pop();                                                                       // 305
    var edges = edgesByRecipient[recipient];                                                               // 306
    if (edges) {                                                                                           // 307
      for (var ii = 0; ii < edges.length; ++ii) {                                                          // 308
        var inEdge = edges[ii];                                                                            // 309
        var sharer = inEdge.sharer;                                                                        // 310
        if (sharer == owner) {                                                                             // 311
          return true;                                                                                     // 312
        }                                                                                                  // 313
        if (!stackedUsers[sharer]) {                                                                       // 314
          userStack.push(sharer);                                                                          // 315
          stackedUsers[sharer] = true;                                                                     // 316
        }                                                                                                  // 317
      }                                                                                                    // 318
    }                                                                                                      // 319
  }                                                                                                        // 320
  return false;                                                                                            // 321
}                                                                                                          // 322
                                                                                                           // 323
SandstormPermissions.downstreamTokens = function(db, root) {                                               // 324
  // Computes a list of the UiView tokens that are downstream in the sharing graph from a given            // 325
  // source. The source, `root`, can either be a token or a (grain, user) pair. The exact format           // 326
  // of `root` is specified in the `check()` invocation below.                                             // 327
  //                                                                                                       // 328
  // TODO(someday): Once UiView tokens can have membrane requirements, we'll need to account for           // 329
  // them in this computation.                                                                             // 330
                                                                                                           // 331
  check(root, Match.OneOf({token: Match.ObjectIncluding({_id: String, grainId: String})},                  // 332
                          {grain: Match.ObjectIncluding({_id: String, identityId: String})}));             // 333
                                                                                                           // 334
  var result = [];                                                                                         // 335
  var tokenStack = [];                                                                                     // 336
  var stackedTokens = {};                                                                                  // 337
  var tokensBySharer = {};                                                                                 // 338
  var tokensByParent = {};                                                                                 // 339
  var tokensById = {};                                                                                     // 340
                                                                                                           // 341
  function addChildren(tokenId) {                                                                          // 342
    var children = tokensByParent[tokenId];                                                                // 343
    if (children) {                                                                                        // 344
      children.forEach(function (child) {                                                                  // 345
        if (!stackedTokens[child._id]) {                                                                   // 346
          tokenStack.push(child);                                                                          // 347
          stackedTokens[child._id] = true;                                                                 // 348
        }                                                                                                  // 349
      });                                                                                                  // 350
    }                                                                                                      // 351
  }                                                                                                        // 352
                                                                                                           // 353
  function addSharedTokens(sharer) {                                                                       // 354
    var sharedTokens = tokensBySharer[sharer];                                                             // 355
    if (sharedTokens) {                                                                                    // 356
      sharedTokens.forEach(function (sharedToken) {                                                        // 357
        if (!stackedTokens[sharedToken._id]) {                                                             // 358
          tokenStack.push(sharedToken);                                                                    // 359
          stackedTokens[sharedToken._id] = true;                                                           // 360
        }                                                                                                  // 361
      });                                                                                                  // 362
    }                                                                                                      // 363
  }                                                                                                        // 364
                                                                                                           // 365
  var grainId;                                                                                             // 366
  if (root.token) {                                                                                        // 367
    grainId = root.token.grainId;                                                                          // 368
  } else if (root.grain) {                                                                                 // 369
    grainId = root.grain._id;                                                                              // 370
  }                                                                                                        // 371
                                                                                                           // 372
  var grain = db.getGrain(grainId);                                                                        // 373
  if (!grain || !grain.private ) { return result; }                                                        // 374
                                                                                                           // 375
  db.collections.apiTokens.find({grainId: grainId,                                                         // 376
                                 revoked: {$ne: true}}).forEach(function (token) {                         // 377
    tokensById[token._id] = token;                                                                         // 378
    if (token.parentToken) {                                                                               // 379
      if (!tokensByParent[token.parentToken]) {                                                            // 380
        tokensByParent[token.parentToken] = [];                                                            // 381
      }                                                                                                    // 382
      tokensByParent[token.parentToken].push(token);                                                       // 383
    } else if (token.identityId) {                                                                         // 384
      if (!tokensBySharer[token.identityId]) {                                                             // 385
        tokensBySharer[token.identityId] = [];                                                             // 386
      }                                                                                                    // 387
      tokensBySharer[token.identityId].push(token);                                                        // 388
    }                                                                                                      // 389
  });                                                                                                      // 390
                                                                                                           // 391
  if (root.token) {                                                                                        // 392
    addChildren(root.token._id);                                                                           // 393
  } else if (root.grain) {                                                                                 // 394
    addSharedTokens(root.grain.identityId);                                                                // 395
  }                                                                                                        // 396
                                                                                                           // 397
  while (tokenStack.length > 0) {                                                                          // 398
    var token = tokenStack.pop();                                                                          // 399
    result.push(token);                                                                                    // 400
    addChildren(token._id);                                                                                // 401
    if (token.owner && token.owner.user) {                                                                 // 402
      addSharedTokens(token.owner.user.identityId);                                                        // 403
    }                                                                                                      // 404
  }                                                                                                        // 405
                                                                                                           // 406
  return result;                                                                                           // 407
}                                                                                                          // 408
                                                                                                           // 409
var HeaderSafeString = Match.Where(function (str) {                                                        // 410
  check(str, String);                                                                                      // 411
  return str.match(/^[\x20-\x7E]*$/);                                                                      // 412
});                                                                                                        // 413
                                                                                                           // 414
var DavClass = Match.Where(function (str) {                                                                // 415
  check(str, String);                                                                                      // 416
  return str.match(/^[a-zA-Z0-9!#$%&'*+.^_`|~-]+$/) ||                                                     // 417
         str.match(/^<[\x21-\x7E]*>$/);  // supposed to be a URL                                           // 418
});                                                                                                        // 419
                                                                                                           // 420
var ResourceMap = Match.Where(function (map) {                                                             // 421
  for (path in map) {                                                                                      // 422
    if (!path.match(/^\/[\x21-\x7E]*$/)) {                                                                 // 423
      return false;                                                                                        // 424
    }                                                                                                      // 425
    check(map[path], {                                                                                     // 426
      type: HeaderSafeString,                                                                              // 427
      language: Match.Optional(HeaderSafeString),                                                          // 428
      encoding: Match.Optional(HeaderSafeString),                                                          // 429
      body: String                                                                                         // 430
    });                                                                                                    // 431
  }                                                                                                        // 432
  return true;                                                                                             // 433
});                                                                                                        // 434
                                                                                                           // 435
var LocalizedString = {                                                                                    // 436
  defaultText: String,                                                                                     // 437
  localizations: Match.Optional([                                                                          // 438
     { locale: String, text: String }                                                                      // 439
  ]),                                                                                                      // 440
};                                                                                                         // 441
                                                                                                           // 442
SandstormPermissions.createNewApiToken = function (db, provider, grainId, petname,                         // 443
                                                   roleAssignment, owner, unauthenticated) {               // 444
  // Creates a new UiView API token. If `rawParentToken` is set, creates a child token.                    // 445
  check(grainId, String);                                                                                  // 446
  check(petname, String);                                                                                  // 447
  check(roleAssignment, db.roleAssignmentPattern);                                                         // 448
  // Meteor bug #3877: we get null here instead of undefined when we                                       // 449
  // explicitly pass in undefined.                                                                         // 450
  check(provider, Match.OneOf({identityId: String, accountId: String},                                     // 451
                              {rawParentToken: Match.OneOf(String, Buffer)}));                             // 452
  check(owner, Match.OneOf({webkey: Match.OneOf(null, {forSharing: Boolean,                                // 453
                                     expiresIfUnusedDuration: Match.Optional(Number)})},                   // 454
                           {user: {identityId: String,                                                     // 455
                                   title: String}},                                                        // 456
                           {grain: {grainId: String,                                                       // 457
                                    saveLabel: LocalizedString,                                            // 458
                                    introducerIdentity: String,}},                                         // 459
                           {frontend: null}));                                                             // 460
  check(unauthenticated, Match.OneOf(undefined, null, {                                                    // 461
    options: Match.Optional({ dav: [Match.Optional(DavClass)] }),                                          // 462
    resources: Match.Optional(ResourceMap),                                                                // 463
  }));                                                                                                     // 464
                                                                                                           // 465
  if (unauthenticated && JSON.stringify(unauthenticated).length > 4096) {                                  // 466
    throw new Meteor.Error(400, "Unauthenticated params too large; limit 4kb.");                           // 467
  }                                                                                                        // 468
                                                                                                           // 469
  var grain = db.getGrain(grainId);                                                                        // 470
  if (!grain) {                                                                                            // 471
    throw new Meteor.Error(403, "Unauthorized", "No grain found.");                                        // 472
  }                                                                                                        // 473
                                                                                                           // 474
  var token = Random.secret();                                                                             // 475
  var apiToken = {                                                                                         // 476
    _id: Crypto.createHash("sha256").update(token).digest("base64"),                                       // 477
    grainId: grainId,                                                                                      // 478
    roleAssignment: roleAssignment,                                                                        // 479
    petname: petname,                                                                                      // 480
    created: new Date(),                                                                                   // 481
    expires: null,                                                                                         // 482
  };                                                                                                       // 483
                                                                                                           // 484
  var parentForSharing = false;                                                                            // 485
  if (provider.rawParentToken) {                                                                           // 486
    var parentToken = Crypto.createHash("sha256").update(provider.rawParentToken).digest("base64");        // 487
    var parentApiToken = db.collections.apiTokens.findOne(                                                 // 488
      {_id: parentToken, grainId: grainId, objectId: {$exists: false}});                                   // 489
    if (!parentApiToken) {                                                                                 // 490
      throw new Meteor.Error(403, "No such parent token found.");                                          // 491
    }                                                                                                      // 492
    if (parentApiToken.forSharing) {                                                                       // 493
      parentForSharing = true;                                                                             // 494
    }                                                                                                      // 495
                                                                                                           // 496
    apiToken.identityId = parentApiToken.identityId;                                                       // 497
    apiToken.accountId = parentApiToken.accountId;                                                         // 498
                                                                                                           // 499
    apiToken.parentToken = parentToken;                                                                    // 500
  } else if (provider.identityId) {                                                                        // 501
    apiToken.identityId = provider.identityId;                                                             // 502
    apiToken.accountId = provider.accountId;                                                               // 503
  }                                                                                                        // 504
                                                                                                           // 505
  if (owner.webkey) {                                                                                      // 506
    // Non-null webkey is a special case not covered in ApiTokenOwner.                                     // 507
    // TODO(cleanup): Maybe ApiTokenOwner.webkey should be extended with these fields?                     // 508
    apiToken.owner = {webkey: null};                                                                       // 509
    apiToken.forSharing = parentForSharing || owner.webkey.forSharing;                                     // 510
    if (owner.webkey.expiresIfUnusedDuration) {                                                            // 511
      apiToken.expiresIfUnused = new Date(Date.now() + owner.webkey.expiresIfUnusedDuration);              // 512
    }                                                                                                      // 513
  } else if (owner.user) {                                                                                 // 514
    var grainInfo = db.getDenormalizedGrainInfo(grainId);                                                  // 515
    apiToken.owner = {                                                                                     // 516
      user: {                                                                                              // 517
        identityId: owner.user.identityId,                                                                 // 518
        title: owner.user.title,                                                                           // 519
        // lastUsed: ??                                                                                    // 520
        denormalizedGrainMetadata: grainInfo,                                                              // 521
      }                                                                                                    // 522
    };                                                                                                     // 523
  } else {                                                                                                 // 524
    // Note: Also covers the case of `webkey: null`.                                                       // 525
    apiToken.owner = owner;                                                                                // 526
  }                                                                                                        // 527
                                                                                                           // 528
  if (unauthenticated) {                                                                                   // 529
    const apiHost = {                                                                                      // 530
      _id: db.apiHostIdHashForToken(token),                                                                // 531
      hash2: Crypto.createHash("sha256").update(apiToken._id).digest("base64"),                            // 532
    };                                                                                                     // 533
    if (unauthenticated.options) {                                                                         // 534
      apiHost.options = unauthenticated.options;                                                           // 535
    }                                                                                                      // 536
    if (unauthenticated.resources) {                                                                       // 537
      // Mongo requires keys in objects to be escaped. Ugh.                                                // 538
      apiHost.resources = {};                                                                              // 539
      for (var key in unauthenticated.resources) {                                                         // 540
        apiHost.resources[SandstormDb.escapeMongoKey(key)] = unauthenticated.resources[key];               // 541
      }                                                                                                    // 542
    }                                                                                                      // 543
                                                                                                           // 544
    db.collections.apiHosts.insert(apiHost);                                                               // 545
    apiToken.hasApiHost = true;                                                                            // 546
  }                                                                                                        // 547
                                                                                                           // 548
  db.collections.apiTokens.insert(apiToken);                                                               // 549
                                                                                                           // 550
  return {id: apiToken._id, token: token, parentApiToken: parentApiToken};                                 // 551
}                                                                                                          // 552
                                                                                                           // 553
// Make self-destructing tokens actually self-destruct, so they don't                                      // 554
// clutter the token list view.                                                                            // 555
SandstormPermissions.cleanupSelfDestructing = function (db) {                                              // 556
  return function () {                                                                                     // 557
    var now = new Date();                                                                                  // 558
    db.removeApiTokens({expiresIfUnused: {$lt: now}});                                                     // 559
  }                                                                                                        // 560
}                                                                                                          // 561
                                                                                                           // 562
Meteor.methods({                                                                                           // 563
  transitiveShares: function(identityId, grainId) {                                                        // 564
    check(identityId, String);                                                                             // 565
    check(grainId, String);                                                                                // 566
    if (this.userId) {                                                                                     // 567
      var db = this.connection.sandstormDb;                                                                // 568
      return SandstormPermissions.downstreamTokens(db,                                                     // 569
          {grain: {_id: grainId, identityId: identityId }});                                               // 570
    }                                                                                                      // 571
  },                                                                                                       // 572
                                                                                                           // 573
  newApiToken: function (provider, grainId, petname, roleAssignment, owner, unauthenticated) {             // 574
    check(provider, Match.OneOf({identityId: String}, {rawParentToken: String}));                          // 575
    // other check()s happen in SandstormPermissions.createNewApiToken().                                  // 576
    var db = this.connection.sandstormDb;                                                                  // 577
    if (provider.identityId) {                                                                             // 578
      if (!this.userId || !db.userHasIdentity(this.userId, provider.identityId)) {                         // 579
        throw new Meteor.Error(403, "Not an identity of the current user: " + provider.identityId);        // 580
      }                                                                                                    // 581
    }                                                                                                      // 582
    if (provider.identityId) {                                                                             // 583
      provider.accountId = this.userId;                                                                    // 584
    }                                                                                                      // 585
    return SandstormPermissions.createNewApiToken(                                                         // 586
      this.connection.sandstormDb, provider, grainId, petname, roleAssignment, owner,                      // 587
      unauthenticated);                                                                                    // 588
  },                                                                                                       // 589
                                                                                                           // 590
  updateApiToken: function (token, newFields) {                                                            // 591
    var db = this.connection.sandstormDb;                                                                  // 592
                                                                                                           // 593
    check(token, String);                                                                                  // 594
    check(newFields, {petname: Match.Optional(String),                                                     // 595
                      roleAssignment: Match.Optional(db.roleAssignmentPattern),                            // 596
                      revoked: Match.Optional(Boolean)});                                                  // 597
                                                                                                           // 598
    if (!this.userId) {                                                                                    // 599
      throw new Meteor.Error(403, "Must be logged in to modify a token");                                  // 600
    }                                                                                                      // 601
    var apiToken = db.collections.apiTokens.findOne(token);                                                // 602
    if (!apiToken) {                                                                                       // 603
      throw new Meteor.Error(404, "No such token found.");                                                 // 604
    }                                                                                                      // 605
                                                                                                           // 606
    if (db.userHasIdentity(this.userId, apiToken.identityId)) {                                            // 607
      var modifier = {$set: newFields};                                                                    // 608
      db.collections.apiTokens.update(token, modifier);                                                    // 609
    } else {                                                                                               // 610
      throw new Meteor.Error(403, "User not authorized to modify this token.");                            // 611
    }                                                                                                      // 612
  }                                                                                                        // 613
});                                                                                                        // 614
                                                                                                           // 615
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['sandstorm-permissions'] = {
  SandstormPermissions: SandstormPermissions
};

})();

//# sourceMappingURL=sandstorm-permissions.js.map
